@@ -35,9 +35,15 @@
 
 #include <sys/socket.h>
 #include <netinet/in.h>
+#include <netinet/ip.h>
+#include <netinet/ip_icmp.h>
 #include <netdb.h>
 #include <stdio.h>
 #include <string.h>
+#include <stdlib.h>
+
+char *ping_buffer = 0;
+size_t ping_pkt_size;
 
 int open_ping_socket_ipv4()
 {
@@ -61,6 +67,15 @@ int open_ping_socket_ipv4()
     return s;
 }
 
+void init_ping_buffer_ipv4(size_t ping_data_size)
+{
+    /* allocate ping buffer */
+    ping_pkt_size = ping_data_size + ICMP_MINLEN;
+    ping_buffer = (char *) calloc(1, ping_pkt_size);
+    if(!ping_buffer)
+        crash_and_burn( "can't malloc ping packet" );
+}
+
 void socket_set_src_addr_ipv4(int s, FPING_INADDR src_addr)
 {
     struct sockaddr_in sa;
@@ -71,3 +86,46 @@ void socket_set_src_addr_ipv4(int s, FPING_INADDR src_addr)
     if ( bind( s, (struct sockaddr *)&sa, sizeof( sa ) ) < 0 )
         errno_crash_and_burn( "cannot bind source address" );
 }
+
+unsigned short calcsum(unsigned short *buffer, int length)
+{
+    unsigned long sum;
+
+    // initialize sum to zero and loop until length (in words) is 0
+    for (sum=0; length>1; length-=2) // sizeof() returns number of bytes, we're interested in number of words
+	sum += *buffer++;	// add 1 word of buffer to sum and proceed to the next
+
+    // we may have an extra byte
+    if (length==1)
+	sum += (char)*buffer;
+
+    sum = (sum >> 16) + (sum & 0xFFFF); // add high 16 to low 16
+    sum += (sum >> 16);			// add carry
+    return ~sum;
+}
+
+int socket_sendto_ping_ipv4(int s, struct sockaddr *saddr, socklen_t saddr_len, uint16_t icmp_seq_nr, uint16_t icmp_id_nr)
+{
+    struct icmp *icp;
+    int n;
+
+    icp = (struct icmp *) ping_buffer;
+
+    icp->icmp_type = ICMP_ECHO;
+    icp->icmp_code = 0;
+    icp->icmp_cksum = 0;
+    icp->icmp_seq = htons(icmp_seq_nr);
+    icp->icmp_id = htons(icmp_id_nr);
+
+    if (random_data_flag) {
+        for (n = ((void*)&icp->icmp_data - (void *)icp); n < ping_pkt_size; ++n) {
+            ping_buffer[n] = random() & 0xFF;
+        }
+    }
+
+    icp->icmp_cksum = calcsum((unsigned short *) icp, ping_pkt_size);
+
+    n = sendto(s, icp, ping_pkt_size, 0, saddr, saddr_len);
+
+    return n;
+}
